require 'spec_helper'

describe Speedflow::Plugin::Jira::Client do
  let(:config_site) { 'https://xxx.atlassian.net/' }
  let(:args) do
    title    = { 'value' => 'test' }
    assignee = { 'default' => 'julien.breux' }
    config   = { 'jira' => { 'site' => config_site } }
    { 'title' => title, 'assignee' => assignee, '_config' => config }
  end
  let(:config) do
    ::Speedflow::Plugin::Configuration.new(args, 'jira')
  end
  let(:prompt) do
    prompt = ::Speedflow::Plugin::Jira::Prompt.new
    prompt.prompt = double
    prompt
  end
  let(:jira_client) { double }
  let(:client) do
    client = ::Speedflow::Plugin::Jira::Client.new(config, prompt)
    client.jira_client = jira_client
    client
  end
  let(:exception_response) do
    double(body: '{"errors": {"k": "v"}, "errorMessages": ["v"]}')
  end

  it '.search_issue' do
    allow(jira_client).to receive_message_chain('Issue.jql')

    client.search_issue('test', 'test')
  end

  it '.create_issue' do
    receive = receive_message_chain('Issue.build') do
      double(save!: nil, fetch: nil)
    end
    allow(jira_client)
      .to receive

    client.create_issue('1', 'test', '1')
  end

  it '.update_issue_assignee' do
    receive = receive_message_chain('Issue.find') do
      double(save!: nil)
    end
    allow(jira_client)
      .to receive

    client.update_issue_assignee('1', 'julien.breux')
  end

  it '.update_issue_transition' do
    receive = receive_message_chain('Issue.find') do
      double(transitions: double(build: double(save!: nil)))
    end
    allow(jira_client)
      .to receive

    client.update_issue_transition('T-1', 1)
  end

  it '.project' do
    allow(jira_client).to receive_message_chain('Project.find')

    client.project('T')
  end

  it '.projects' do
    allow(jira_client).to receive_message_chain('Project.all')

    client.projects
  end

  it '.issue_types' do
    allow(jira_client).to receive_message_chain('Issuetype.all')

    client.issue_types
  end

  it '.issue_trans' do
    receive = receive_message_chain('Issue.find') do
      double(transitions: nil)
    end
    allow(jira_client).to receive

    client.issue_trans('T-1')
  end

  it '.issue_link' do
    expect(client.issue_link('T-1'))
      .to eq config_site
  end

  it '.issue_output_format' do
    issue = double(key: 'T-1', fields: { 'summary' => 'Test' })

    expected_hash = {
      'issue' => { 'key' => 'T-1', 'summary' => 'Test',
                   'url' => "#{config_site}browse/T-1" } }
    expect(client.issue_output_format(issue))
      .to eq expected_hash
  end

  it '.safe' do
    allow(client.prompt)
      .to receive(:error).with('Invalid URL')
    e = expect do
      client.safe do
        raise ::URI::InvalidURIError.new(''), '...'
      end
    end
    e.to raise_error(SystemExit)

    allow(client.prompt)
      .to receive(:error).with('Jira errors')
    allow(client.prompt)
      .to receive(:errors)
    e = expect do
      client.safe do
        raise ::JIRA::HTTPError.new(exception_response), '...'
      end
    end
    e.to raise_error(SystemExit)
  end

  it '.auth' do
    expected_hash = {
      auth_type:    :basic,
      context_path: '',
      password:     '',
      read_timeout: 120,
      site:         'https://xxx.atlassian.net/',
      username:     ''
    }

    expect(client.auth)
      .to eq expected_hash
  end
end
